package checkAlign;

import java.awt.event.*;
import javax.swing.*;
import java.io.*;

import javax.swing.JFileChooser;

public class Controller {
	
	Model model;
	View view;
	File2String f2s;
	
    public Controller(Model model, View view){
        this.model = model;
        this.view = view;
        
        addListeners();        
    }
    
    private void addListeners(){
    	view.addLoadListener(new LoadListener());
    	view.addSaveListener(new SaveListener());
    	view.addAboutListener(new AboutListener());
    	view.addTestListener(new TestListener());
    	view.addExitListener(new ExitListener());
    }
    
    class LoadListener implements ActionListener{
    	public void actionPerformed(ActionEvent ae){
    		openFile();
    	}
    }
    class SaveListener implements ActionListener{
    	public void actionPerformed(ActionEvent ae){
    		saveFile();
    	}
    }
    class AboutListener implements ActionListener{
    	public void actionPerformed(ActionEvent ae){
    		about();
    	}
    }
    class ExitListener implements ActionListener{
    	public void actionPerformed(ActionEvent ae){
    		exitProg();
    	}    	
    }
    class TestListener implements ActionListener{
    	public void actionPerformed(ActionEvent ae){
    		test();
    	}
    }
    
    private void openFile(){
    	JFileChooser chooser = new JFileChooser();
    	int res = chooser.showOpenDialog(view);
    	if (res == JFileChooser.APPROVE_OPTION){
    		File file = chooser.getSelectedFile();
    		f2s = new File2String(file);
    		//model.setAlignment(f2s.getString());
    		view.textAlign.setText(f2s.getString());
    		
    		// detect illegal chars
    		/*
    		if (model.hasIllegalChars()){
    			JOptionPane.showMessageDialog(null, "The alignment has illegal chars", "Warning", JOptionPane.WARNING_MESSAGE);
    		}
    		*/
    	}else if (res == JFileChooser.CANCEL_OPTION){
    		return;
    	}
    	view.labelStatus.setText("Select a method");
    }
    private void saveFile(){
    	// must check if an alignment is selected
    	if (view.textAlign.getText().compareTo("") == 0){
    		JOptionPane.showMessageDialog(view, "Alignment not loaded", "Message", JOptionPane.WARNING_MESSAGE);
    		return;
    	}
    	
    	if (view.radioMethod2.isSelected()){
    		try{
	    		double valFreq = Double.parseDouble(view.checkMethod2Freq.getText());
	    		if (valFreq < 0 || valFreq > 100){
	    			JOptionPane.showMessageDialog(view, "Frequency not valid", "Message", JOptionPane.WARNING_MESSAGE);
	        		return;
	    		}
    		}catch(Exception ex){
    			JOptionPane.showMessageDialog(view, "Frequency not valid", "Message", JOptionPane.WARNING_MESSAGE);
        		return;
    		}
    	}
    		
    	// must get interface variables	 
    	int method = 1;    	
    	boolean gaps = view.checkMethod1Gaps.isSelected();
    	boolean correction = view.checkMethod1Correction.isSelected();
    	boolean sd = view.checkMethod1Sd.isSelected();
    	boolean protein = false;
    	String percent = view.checkMethod2Freq.getText();
    	String psFiles[];
    	
    	if (view.radioMethod1.isSelected()) method = 1;
    	else if (view.radioMethod2.isSelected()) method = 2;
    	
    	if (view.radioProtein.isSelected()) protein = true;
    	
    	// create matrix
    	model.setAlignment(f2s.getString(), protein);
    	
    	// perform calulations
    	model.calculate(method, protein, gaps, correction, sd, Integer.parseInt(percent));
    	
    	// create ps files
    	psFiles = model.getPsFiles();    	
    	
    	// save file to hard disk    	
    	JFileChooser chooser = new JFileChooser();
    	File output;
    	String outputPath = "";
    	FileWriter fw;
    	BufferedWriter bw;
    	
    	chooser.setCurrentDirectory(new File("./postscript"));
    	
    	int res = chooser.showSaveDialog(view);
    	
    	if (res == JFileChooser.APPROVE_OPTION){
    		try{
    		    output = chooser.getSelectedFile();
    		    outputPath = output.getPath();
    		    
    		    if (outputPath.endsWith(".ps")){
    		    	// remove extension if exists
    		    	outputPath = outputPath.substring(0, outputPath.lastIndexOf(".ps"));
    		    }
    		    
    		    for (int i=0; i<psFiles.length; i++){
    		    	 // extension must be .ps  
    		    	String outputPath2 = outputPath+"_"+i+".ps";
    		    	
    		    	if (psFiles.length == 1){
    		    		outputPath2 = outputPath+".ps";
    		    	}
    		    	
    		    	output = new File(outputPath2);
    		    	
        		    fw = new FileWriter(output);
        		    bw = new BufferedWriter(fw);
        		    bw.write(psFiles[i]);
        		    bw.close();
        		    fw.close();
    		    }
    		    view.labelStatus.setText("File saved: "+chooser.getSelectedFile());
    		}catch (Exception ex){
    		    ex.printStackTrace();
    		}
    		
    	}else if (res == JFileChooser.CANCEL_OPTION){
    		return;
    	}
    }
    private void about(){
    	String message1 = "Check align v1.0\nby Biotech Vana\nhttp://www.biotechvana.org";
    	JOptionPane.showMessageDialog(null, message1, "About...", JOptionPane.INFORMATION_MESSAGE);
    }
    private void exitProg(){
    	System.exit(0);
    }
    private void test(){
    	File2String f2s = new File2String(new File("resources/ab.ps"));
    	
    	ExportPng ep = new ExportPng(f2s.getString());
    	ep.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);    	
    	ep.pack();
    	ep.setVisible(true);
    }
}
